local computer = require("computer")
local event = require("event")

local bus = {}

bus.data = {}
bus.info = {}

local db = {}

local function tab_contains(table, value)
 for k, v in pairs(table) do
  if v == value then
   return true, k
  end
 end
 return false
end

local function update_timed(name)
 local now = computer.uptime()
 for i, d in pairs(bus.data[name]) do
  local t = bus.info[name].time_info[i]
  if now - t > bus.info[name].timeout then
   computer.pushSignal("data_list_remove", name, d)
   computer.pushSignal("data_update", name)
   bus.data[name][i] = nil
   bus.info[name].time_info[i] = nil
  end
 end
end

function db.setupValue(name, props)
 if props.type == "normal" then
  bus.info[name] = props
 elseif props.type == "timed_list" then
  bus.info[name] = {type="timed_list", timeout=props.timeout or 30, allow_duplicates=props.allow_duplicates , time_info = {} }
  if bus.info[name].allow_duplicates == nil then
   bus.info[name].allow_duplicates = false
  end
  bus.data[name] = {}
 end
end

function db.push(name, data)
 if bus.info[name].type == "timed_list" then
  if bus.info[name].allow_duplicates or not tab_contains(bus.data[name], data) then
   local n = math.random(1000000)
   while bus.data[name][n] ~= nil do n = math.random(1000000) end
   bus.data[name][n] = data
   bus.info[name].time_info[n] = computer.uptime()
   computer.pushSignal("data_list_add", name, data)
   computer.pushSignal("data_update", name)
  else
   local r, n = tab_contains(bus.data[name], data)
   bus.info[name].time_info[n] = computer.uptime()
  end
 else
  local new = bus.data[name] ~= data
  bus.data[name] = data
  if new then
   computer.pushSignal("data_update", name)
  end
 end
end

function db.checkout(name, update_data)
 if update_data == nil then
  update_data = true
 end
 if bus.info[name].type == "timed_list" then
  if update_data == true then
   update_timed(name)
  end
  local d = {}
  for _, v in pairs(bus.data[name]) do
   table.insert(d, v)
  end  
  return d
 else
  return bus.data[name] 
 end
end

local n = event.timer(1, function()
 for k, v in pairs(bus.info) do
  if v.type == "timed_list" then
   update_timed(k)
  end
 end
end, math.huge)

return db;